import React, { Component } from 'react'
import { connect } from 'react-redux'
import { Redirect } from 'react-router-dom'
import Paper from 'material-ui/Paper'
import Typography from 'material-ui/Typography'
import TextField from 'material-ui/TextField'
import Button from 'material-ui/Button'
import Snackbar from 'material-ui/Snackbar'
import { AUTH } from '../../constants/ResourceNames'
import { post, notify } from '../../actions'
import { getToken } from '../../utils/auth'
import { withStyles } from 'material-ui/styles'

const styles = theme => ({
    root: {
        position: 'absolute',
        width: '100%',
        height: '100%',
        display: 'flex',
        alignItems: 'center',
        justifyContent: 'center'
    },
    paper: {
        padding: theme.spacing.unit * 3
    },
    formHeading: {
        marginTop: theme.spacing.unit * 3,
        marginBottom: theme.spacing.unit * 2
    },
    form: {
        display: 'flex',
        flexDirection: 'column',
        width: 350,
        height: 230
    },
    txtField: {
        margin: theme.spacing.unit * 2
    },
    submitBtn: {
        marginTop: theme.spacing.unit * 3
    }
})

class Login extends Component {
    /**
     * 
     * @param {*} props 
     */
    constructor(props) {
        super(props)

        this.state = {
            username: '',
            password: '',
            hasError: false,
            redirectToReferrer: false
        }
    }

    /**
     * 
     */
    componentWillMount() {
        this.props.check()
    }

    /**
     * 
     */
    handleChangeInput = e => {
        this.setState({
            [e.target.id]: e.target.value
        })
    }

    /**
     * 
     */
    handleSubmit = e => {
        const { username, password } = this.state

        this.props.submit(e, username, password)

        this.setState({
            username: '',
            password: ''
        }, this.handleFormReset)
    }

    /**
     * 
     */
    handleFormReset = () => {
        this.authForm.reset()
    }

    /**
     * 
     */
    render() {
        const { from } = this.props.location.state || { from: { pathname: '/' }}
        const { isAuthenticated, notification, onHideNotification, classes } = this.props
        const { hasError } = this.state

        if (isAuthenticated) {
            return <Redirect to={from} />
        }

        return (
            <div className={classes.root}>
                <Paper className={classes.paper} elevation={4}>
                    <Typography className={classes.formHeading} variant='display1' color='primary' align='center'>Eiru Automation</Typography>
                    <form className={classes.form} noValidate autoComplete='off' ref={el => this.authForm = el}>
                        <TextField
                            id='username'
                            className={classes.txtField} 
                            error={hasError}
                            onKeyUp={this.handleChangeInput}
                            label='Usuario'
                            autoFocus 
                        />
                        <TextField 
                            id='password'
                            className={classes.txtField} 
                            error={hasError}
                            onKeyUp={this.handleChangeInput}
                            label='Contraseña' 
                            type='password' 
                        />
                        <Button className={classes.submitBtn} variant='raised' color='primary' onClick={this.handleSubmit}>Acceder</Button>
                    </form>
                </Paper>
                <Snackbar 
                    open={notification.isVisible} 
                    autoHideDuration={2000} 
                    onClose={onHideNotification} 
                    message={notification.message} 
                />
            </div>
        )
    }
}

/**
 * 
 * @param {*} state 
 * @param {*} props 
 */
const mapStateToProps = (state, props) => {
    return {
        isAuthenticated: state.auth.isAuthenticated,
        notification: state.notification
    }
}

/**
 * 
 * @param {*} dispatch 
 * @param {*} props 
 */
const mapDispatchToProps = (dispatch, props) => ({
    /**
     * 
     */
    check() {
        const token = getToken()

        if (!token) {
            return
        }

        dispatch(post(`${AUTH}check_token/`, { token }))
    },
    /**
     * 
     * @param {*} e 
     * @param {*} username 
     * @param {*} password 
     */
    submit(e, username, password) {
        dispatch(post(`${AUTH}get_token/`, { username, password }))
    },
    /**
     * 
     */
    onHideNotification() {
        dispatch(notify(null))
    },
})

Login = withStyles(styles)(Login)

export default connect(mapStateToProps, mapDispatchToProps)(Login)