import { createReducer } from '../utils/reducer'
import { REQUEST_OK, REQUEST_KO, HIDE_NOTIFICATION } from '../constants/ActionTypes'
import { has, isEqual } from 'lodash'

const initialState = {
    isVisible: false,
    message: null
}

/**
 * 
 * @param {*} state 
 * @param {*} action 
 */
const showNotification = (state, action) => {
    if (!has(action.payload, 'status')) {
        return {
            isVisible: true,
            message: action.payload.message
        }
    }

    if (isEqual(action.payload.status, 401)) {
        if (has(action.payload, 'error_message')) {
            return {
                isVisible: true,
                message: 'Fallo en la autenticación del usuario'
            }
        }

        return state
    }

    return state
}

/**
 * 
 * @param {*} state 
 * @param {*} action 
 */
const hideNotification = (state, action) => {
    return {
        isVisible: false,
        message: null
    }
}

/**
 * 
 */
const notificationReducer = createReducer(initialState, {
    [REQUEST_OK]: showNotification,
    [REQUEST_KO]: showNotification,
    [HIDE_NOTIFICATION]: hideNotification
})

/**
 * 
 * @param {*} state 
 * @param {*} action 
 */
export const notification = (state = initialState, action) => {
    return notificationReducer(state, action)
}